"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from acuvity.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ExtractionsummaryTypedDict(TypedDict):
    r"""Represents the summary of the extractions."""

    pi_is: NotRequired[Dict[str, Any]]
    r"""The PIIs found during classification."""
    categories: NotRequired[Dict[str, Any]]
    r"""The categories are remapping of the modalities in a more human friendly way."""
    exploits: NotRequired[Dict[str, Any]]
    r"""The various exploits attempts."""
    intent: NotRequired[Dict[str, Any]]
    r"""The estimated intent embodied into the text."""
    keywords: NotRequired[Dict[str, Any]]
    r"""The keywords found during classification."""
    languages: NotRequired[Dict[str, Any]]
    r"""The language of the classification."""
    malcontents: NotRequired[Dict[str, Any]]
    r"""The various malcontents attempts."""
    modalities: NotRequired[Dict[str, Any]]
    r"""The modalities of data detected in the data."""
    secrets: NotRequired[Dict[str, Any]]
    r"""The secrets found during classification."""
    topics: NotRequired[Dict[str, Any]]
    r"""The topic of the classification."""


class Extractionsummary(BaseModel):
    r"""Represents the summary of the extractions."""

    pi_is: Annotated[Optional[Dict[str, Any]], pydantic.Field(alias="PIIs")] = None
    r"""The PIIs found during classification."""

    categories: Optional[Dict[str, Any]] = None
    r"""The categories are remapping of the modalities in a more human friendly way."""

    exploits: Optional[Dict[str, Any]] = None
    r"""The various exploits attempts."""

    intent: Optional[Dict[str, Any]] = None
    r"""The estimated intent embodied into the text."""

    keywords: Optional[Dict[str, Any]] = None
    r"""The keywords found during classification."""

    languages: Optional[Dict[str, Any]] = None
    r"""The language of the classification."""

    malcontents: Optional[Dict[str, Any]] = None
    r"""The various malcontents attempts."""

    modalities: Optional[Dict[str, Any]] = None
    r"""The modalities of data detected in the data."""

    secrets: Optional[Dict[str, Any]] = None
    r"""The secrets found during classification."""

    topics: Optional[Dict[str, Any]] = None
    r"""The topic of the classification."""



from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

import yaml

from .constants import GuardName
from .errors import (
    GuardConfigError,
    GuardConfigValidationError,
    GuardThresholdParsingError,
)
from .threshold import DEFAULT_THRESHOLD, Threshold


@dataclass(frozen=True)
class Match:
    """Immutable match configuration"""
    threshold: Threshold
    redact: bool = False
    count_threshold: int = 0

    @classmethod
    def create(
        cls,
        threshold: str | Threshold = DEFAULT_THRESHOLD,
        redact: bool = False,
        count_threshold: int = 0
    ) -> "Match":
        """
        Create a Match with type hints for better IDE support.

        Args:
            threshold: Threshold as string (e.g. '>= 0.8') or Threshold object
            redact: Whether to redact matches
            count_threshold: Minimum count threshold
        """
        if isinstance(threshold, str):
            threshold = Threshold(threshold)

        return cls(
            threshold=threshold,
            redact=redact,
            count_threshold=count_threshold
        )

@dataclass(frozen=True)
class Guard:
    """Immutable guard configuration"""
    name: GuardName
    matches: Dict[str, Match]
    threshold: Threshold
    count_threshold: int = 0

    def __post_init__(self):
        if not isinstance(self.name, GuardName):
            if not isinstance(self.name, str):
                # if the guardname is not a guardname enum and not a str.
                raise GuardConfigValidationError(
                f"Guard name must be string or GuardName enum, got {type(self.name)}")

            # here its a str but if its not part of the enum then raise a error.
            if GuardName.get(self.name) is None:
                valid_names = ", ".join(GuardName.values())
                raise GuardConfigValidationError(
                    f"Invalid guard name: {self.name}. Must be one of: {valid_names}"
                )

        # Validate threshold
        if isinstance(self.threshold, str):
            try:
                _ = Threshold(self.threshold)
            except GuardConfigValidationError as e:
                raise GuardConfigValidationError("Invalid threshold") from e

        if self.count_threshold < 0:
            raise GuardConfigValidationError("Invalid count threshold, should be a positive number")

    @classmethod
    def create(
        cls,
        name: str | GuardName,
        matches: Dict[str, Match] | None = None,
        threshold: str | Threshold = DEFAULT_THRESHOLD,
        count_threshold: int = 0
    ) -> "Guard":
        """
        Create a Guard with type hints for better IDE support.

        Args:
            name: Guard name (string or GuardName enum)
            matches: Dictionary of match configurations
            threshold: Threshold as string (e.g. '>= 0.8') or Threshold object
            count_threshold: Minimum count threshold

        Raises:
            GuardConfigValidationError: If guard name is invalid
        """
        # Convert string to GuardName if needed
        if isinstance(name, str):
            guard_name = GuardName.get(name)
            if guard_name is None:
                valid_names = ", ".join(GuardName.values())
                raise GuardConfigValidationError(
                    f"Invalid guard name: {name}. Must be one of: {valid_names}"
                )
        else:
            guard_name = name

        if isinstance(threshold, str):
            threshold = Threshold(threshold)

        return cls(
            name=guard_name,
            matches=matches or {},
            threshold=threshold,
            count_threshold=count_threshold
        )

class GuardConfig:
    """
    Parser for guard configuration files.

    This class handles parsing of guard configuration files in YAML format,
    validating their contents, and converting between analyzer names and IDs.

    The parser handles two types of guards:
    1. Match Guards: Guards with a 'matches' section (e.g., pii_detector)
    2. Simple Guards: Guards without matches (e.g., prompt_injection, toxic)
    """

    def __init__(self, config: Optional[Union[str, Path, Dict, List[Guard]]] = None):
        """
        Initialize parser with analyzer mapping.

        Args:
            config: Configuration as a string, filepath or dictionary
        """
        self.guards: List[Guard] = []

        # Handle default configuration
        if config is None:
            for guard in GuardName:
                # skip keyword detector in default
                if guard != GuardName.KEYWORD_DETECTOR:
                    self.guards.append(Guard(
                        name=guard,
                        matches={},
                        threshold=DEFAULT_THRESHOLD,
                        count_threshold=0,
                    ))
            return

        # Use the config provided
        self._parse_config(config)

    @staticmethod
    def load_yaml(path: Union[str, Path]) -> Dict[str, Any]:
        """
        Load and parse YAML file.

        Args:
            path: Path to YAML file

        Returns:
            Parsed YAML content as dictionary

        Raises:
            GuardConfigError: If file cannot be read or parsed
        """
        try:
            with open(path, encoding='utf-8') as yaml_file:
                return yaml.safe_load(yaml_file)
        except (yaml.YAMLError, OSError) as e:
            raise GuardConfigError(f"Failed to load config file: {e}") from e

    def _parse_config(self, config: Union[str, Path, Dict, List[Guard]]) -> List[Guard]:
        """
        Parse guard configuration from file or dictionary.

        Args:
            config: Path to YAML file or dictionary containing configuration

        Returns:
            List of parsed Guard objects

        Raises:
            GuardConfigError: If configuration is invalid
        """
        # Handle list of guard dictionaries
        if isinstance(config, list) and all(isinstance(guard, Guard) for guard in config):
            self.guards = [self._parse_guard_obj(guard) for guard in config if self._validate_guard(guard)]
            return self.guards

        config_data: Dict[str, Any]
        if isinstance(config, (str, Path)):
            config_data = self.load_yaml(config)
        elif isinstance(config, dict):
            config_data = config
        else:
            raise ValueError(f"Unexpected config type: {type(config)}")

        try:
            # Handle both single guard and multiple guardrails format
            guards = config_data.get('guardrails', [config_data])
            if not isinstance(guards, list):
                guards = [guards]

            self.guards = [self._parse_guard(guard) for guard in guards
                              if self._validate_guard(guard)]
            return self.guards

        except Exception as e:
            raise GuardConfigError(f"Failed to parse config: {e}") from e

    def _validate_guard(self, guard: Union[Dict, Guard]) -> bool:
        """
        Validate individual guard configuration.

        Args:
            guard: Guard configuration dictionary

        Returns:
            True if guard is valid

        Raises:
            GuardConfigValidationError: If guard configuration is invalid
        """
        if isinstance(guard, Guard) and not GuardName.valid(str(guard.name)):
            raise GuardConfigValidationError("Guard must have a valid name")
        elif isinstance(guard, Dict):
            if 'name' not in guard:
                raise GuardConfigValidationError(f"Guard must have a name, but give guard is: {guard}")

            if not GuardName.valid(guard['name']):
                raise GuardConfigValidationError(f"Guard name not present {guard['name']}")

        return True

    def parse_match_obj(self, match_key: str, match_obj: Match | None) -> Match:
        """
        Parse an existing Match object, applying validation and defaults if needed.

        Args:
            match_key: Key identifying the match
            match_obj: Match object to parse, or None to use defaults

        Returns:
            Match: Validated Match object with defaults applied if needed

        Raises:
            GuardConfigValidationError: If threshold validation fails
        """
        # Handle None case
        if match_obj is None:
            return Match.create()

        try:
            # Use existing values or defaults
            threshold = match_obj.threshold or DEFAULT_THRESHOLD

            return Match.create(
                threshold=threshold,
                redact=match_obj.redact,
                count_threshold=match_obj.count_threshold
            )

        except ValueError as e:
            raise GuardConfigValidationError(
                f"Invalid match configuration for '{match_key}': {str(e)}"
            ) from e

    def _parse_match(self, match_key: str, match_data: Dict) -> Match:
        """
        Parse match configuration.

        Args:
            match_key: Key identifying the match
            match_data: Match configuration dictionary

        Returns:
            Match object
        """
        threshold = DEFAULT_THRESHOLD
        redact = False
        if match_data:
            if 'threshold' in match_data:
                try:
                    threshold = Threshold(match_data['threshold'])
                except GuardConfigValidationError as e:
                    raise GuardConfigValidationError(f"Invalid threshold for match {match_key}") from e

            if 'redact' in match_data:
                redact = match_data['redact']

            return Match(
                threshold=threshold,
                redact=redact,
                count_threshold=match_data.get('count_threshold', 0)
            )

        return Match(
            threshold=threshold,
            redact= redact,
            count_threshold=0
        )

    @property
    def guard_names(self) -> List[GuardName]:
        """
        Returns the list of all guards configured
        """
        names = []
        for guard in self.guards:
            names.append(guard.name)
        return names

    @property
    def redaction_keys(self) -> List[str]:
        """
        Returns the list of the keys that have redaction set.
        """
        redact_keys = []
        for guard in self.guards:
            if guard.matches is None:
                continue
            for key, matches in guard.matches.items():
                if matches.redact:
                    redact_keys.append(key)
        return redact_keys

    @property
    def keywords(self) -> List[str]:
        """
        Returns the list of the keys that have redaction set.
        """
        keywords: List[str] = []
        for guard in self.guards:
            if guard.name == GuardName.KEYWORD_DETECTOR:
                for key, _ in guard.matches.items():
                    keywords.append(key)
        return keywords

    def _parse_guard(self, guard: Dict) -> Guard:
        """
        Parse individual guard configuration.

        Args:
            guard: Guard configuration dictionary

        Returns:
            Guard object

        Raises:
            GuardConfigValidationError: If guard configuration is invalid
        """
        name = guard['name']


        # Parse top-level threshold
        threshold = DEFAULT_THRESHOLD
        if 'threshold' in guard:
            try:
                threshold = Threshold(guard.get('threshold', '>= 0.0'))
            except GuardConfigValidationError as e:
                raise e

        if 'count_threshold' in guard and guard['count_threshold'] > 0 and not guard.get('matches'):
            raise GuardConfigValidationError("Failed to parse Guard object, cannot have count_threshold without matches.")

        # Parse matches
        matches = {}
        for match_key, match_data in guard.get('matches', {}).items():
            matches[match_key] = self._parse_match(match_key, match_data)

        guard_name = GuardName.get(name)
        if not guard_name:
            raise ValueError(f"Invalid guard name, must be one of the {GuardName.values()}")

        return Guard(
                name=guard_name,
                matches=matches,
                threshold=threshold,
                count_threshold=guard.get('count_threshold', 0)
            )

    def _parse_guard_obj(self, guard: Guard) -> Guard:
        """
        Parse (or re-validate) an existing Guard object.

        Args:
            guard: An already-instantiated Guard object.

        Returns:
            A Guard object (either the same or a new one) that has been
            re-validated, re-initialized, or finalized.

        Raises:
            ConfigValidationError: If guard configuration is invalid.
        """
        try:
            # Convert guard.name to GuardName if it's not already
            guard_name = guard.name
            if not isinstance(guard_name, GuardName):
                # If .get() can raise an error, it'll be caught by this try block
                guard_name = GuardName.get(guard_name)

            # Parse or confirm the threshold
            threshold = guard.threshold if guard.threshold is not None else DEFAULT_THRESHOLD
            # If threshold parsing can fail, wrap it similarly:
            # threshold = Threshold(guard.threshold)

            if guard.count_threshold > 0 and len(guard.matches) == 0:
                raise GuardConfigValidationError("Failed to parse Guard object, cannot have count_threshold without matches.")

            # Re-parse or validate each match
            parsed_matches = {}
            for match_key, match_data in guard.matches.items():
                # _parse_match may raise various exceptions if the data is invalid
                parsed_matches[match_key] = self.parse_match_obj(match_key, match_data)

            # Create and return a new Guard object or update the existing one
            return Guard(
                name=guard.name,
                matches=parsed_matches,
                threshold=threshold,
                count_threshold=guard.count_threshold or 0
            )

        except (KeyError, GuardThresholdParsingError, ValueError) as e:
            # Catch whatever errors might arise (KeyError for missing fields, etc.)
            raise GuardConfigValidationError(f"Failed to parse Guard object: {e}") from e



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from acuvity.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DetectorTypedDict(TypedDict):
    r"""Represent a detector an analyzer can use."""

    description: NotRequired[str]
    r"""The description of the detection."""
    group: NotRequired[str]
    r"""The group the detection belongs to."""
    label: NotRequired[str]
    r"""The label returned by the model."""
    name: NotRequired[str]
    r"""The name of the detection."""
    positional: NotRequired[bool]
    r"""Tell if the detection is positional."""


class Detector(BaseModel):
    r"""Represent a detector an analyzer can use."""

    description: Optional[str] = None
    r"""The description of the detection."""

    group: Optional[str] = None
    r"""The group the detection belongs to."""

    label: Optional[str] = None
    r"""The label returned by the model."""

    name: Optional[str] = None
    r"""The name of the detection."""

    positional: Optional[bool] = None
    r"""Tell if the detection is positional."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from acuvity.types import BaseModel
from typing_extensions import TypedDict


class ModalityTypedDict(TypedDict):
    r"""Represents the modality of a some data."""

    group: str
    r"""The group of data.

    The current list can be obtained through the analyzers API by searching for
    detector groups in the 'Modality' analyzer group.

    Example of group: application, archive, audio, code, document, executable, font,
    image, text, unknown, video.
    """
    type: str
    r"""The type of data.

    The current list can be obtained through the analyzers API by searching for
    detector names in the 'Modality' analyzer group.

    Example of type: 3dsm, 3gp, 3mf, ace, ada, ai, apk, applebplist, appleplist, ar,
    arc, arj, asc, asf, asm, asp, au, autohotkey, autoit, avi, avif, awk, ax, batch,
    bazel, bcad, bib, bmp, bpg, brainfuck, brf, bzip, bzip3, c, cab, cat, cdf, chm,
    clojure, cmake, cobol, coff, coffeescript, com, cpl, cpp, crt, crx, cs, csproj,
    css, csv, dart, deb, dex, dey, dicom, diff, django, dll, dm, dmg, dmigd,
    dmscript, doc, dockerfile, docx, dotx, dwg, dxf, dylib, elf, elixir, emf, eml,
    epub, erb, erlang, exe, flac, flv, fortran, fpx, gemfile, gemspec, gif,
    gitattributes, gitmodules, gleam, go, gradle, groovy, gzip, h, h5, handlebars,
    haskell, hcl, heif, hlp, hpp, hta, htaccess, html, hwp, icns, ico, ics,
    ignorefile, ini, internetshortcut, ipynb, iso, jar, java, javabytecode,
    javascript, jinja, jng, jnlp, jp2, jpeg, json, jsonl, jsx, julia, jxl, ko,
    kotlin, latex, lha, license, lisp, lnk, lock, lua, lz, lz4, m3u, m4, macho,
    makefile, markdown, matlab, mht, midi, mkv, mp3, mp4, mpegts, mscompress, msi,
    msix, mui, mum, npy, npz, objectivec, ocaml, ocx, odex, odin, odp, ods, odt,
    ogg, one, onnx, otf, outlook, pascal, pcap, pdb, pdf, pebin, pem, perl, php,
    pickle, png, po, postscript, powershell, ppt, pptx, prolog, proteindb, proto,
    psd, python, pythonbytecode, pytorch, qoi, qt, r, randomascii, rar, rdf, rlib,
    rll, rpm, rst, rtf, ruby, rust, scala, scheme, scr, scss, sevenzip, sgml, shell,
    smali, snap, so, solidity, sql, sqlite, squashfs, srt, stlbinary, stltext, sum,
    svg, swf, swift, symlinktext, sys, tar, tcl, textproto, tga, thumbsdb, tiff,
    toml, torrent, tsv, tsx, ttf, twig, txt, txtascii, txtutf16, txtutf8,
    typescript, vba, vbe, vcxproj, verilog, vhdl, visio, vtt, vue, wad, wasm, wav,
    webm, webp, webtemplate, winregistry, wma, wmf, wmv, woff, woff2, xar, xcf, xls,
    xlsb, xlsx, xml, xpi, xz, yaml, yara, zig, zip, zlibstream, zst.
    """


class Modality(BaseModel):
    r"""Represents the modality of a some data."""

    group: str
    r"""The group of data.

    The current list can be obtained through the analyzers API by searching for
    detector groups in the 'Modality' analyzer group.

    Example of group: application, archive, audio, code, document, executable, font,
    image, text, unknown, video.
    """

    type: str
    r"""The type of data.

    The current list can be obtained through the analyzers API by searching for
    detector names in the 'Modality' analyzer group.

    Example of type: 3dsm, 3gp, 3mf, ace, ada, ai, apk, applebplist, appleplist, ar,
    arc, arj, asc, asf, asm, asp, au, autohotkey, autoit, avi, avif, awk, ax, batch,
    bazel, bcad, bib, bmp, bpg, brainfuck, brf, bzip, bzip3, c, cab, cat, cdf, chm,
    clojure, cmake, cobol, coff, coffeescript, com, cpl, cpp, crt, crx, cs, csproj,
    css, csv, dart, deb, dex, dey, dicom, diff, django, dll, dm, dmg, dmigd,
    dmscript, doc, dockerfile, docx, dotx, dwg, dxf, dylib, elf, elixir, emf, eml,
    epub, erb, erlang, exe, flac, flv, fortran, fpx, gemfile, gemspec, gif,
    gitattributes, gitmodules, gleam, go, gradle, groovy, gzip, h, h5, handlebars,
    haskell, hcl, heif, hlp, hpp, hta, htaccess, html, hwp, icns, ico, ics,
    ignorefile, ini, internetshortcut, ipynb, iso, jar, java, javabytecode,
    javascript, jinja, jng, jnlp, jp2, jpeg, json, jsonl, jsx, julia, jxl, ko,
    kotlin, latex, lha, license, lisp, lnk, lock, lua, lz, lz4, m3u, m4, macho,
    makefile, markdown, matlab, mht, midi, mkv, mp3, mp4, mpegts, mscompress, msi,
    msix, mui, mum, npy, npz, objectivec, ocaml, ocx, odex, odin, odp, ods, odt,
    ogg, one, onnx, otf, outlook, pascal, pcap, pdb, pdf, pebin, pem, perl, php,
    pickle, png, po, postscript, powershell, ppt, pptx, prolog, proteindb, proto,
    psd, python, pythonbytecode, pytorch, qoi, qt, r, randomascii, rar, rdf, rlib,
    rll, rpm, rst, rtf, ruby, rust, scala, scheme, scr, scss, sevenzip, sgml, shell,
    smali, snap, so, solidity, sql, sqlite, squashfs, srt, stlbinary, stltext, sum,
    svg, swf, swift, symlinktext, sys, tar, tcl, textproto, tga, thumbsdb, tiff,
    toml, torrent, tsv, tsx, ttf, twig, txt, txtascii, txtutf16, txtutf8,
    typescript, vba, vbe, vcxproj, verilog, vhdl, visio, vtt, vue, wad, wasm, wav,
    webm, webp, webtemplate, winregistry, wma, wmf, wmv, woff, woff2, xar, xcf, xls,
    xlsb, xlsx, xml, xpi, xz, yaml, yara, zig, zip, zlibstream, zst.
    """



from dataclasses import dataclass, field
from enum import Enum
from typing import List

from acuvity.guard.constants import GuardName


class ResponseMatch(str, Enum):
    """Enumeration for check matches."""
    YES = "YES"
    NO = "NO"

@dataclass
class GuardMatch:
    """Result of a single check operation."""
    response_match: ResponseMatch
    guard_name: GuardName
    actual_value: float
    threshold: str
    match_count: int = 0
    match_values: List[str]= field(default_factory=list)

    def to_dict(self):
        return {
            'guard_name': str(self.guard_name),  # assuming GuardName needs str conversion
        }

@dataclass
class Matches:
    """Result of processing multiple checks or a configuration."""
    input_data: str
    response_match: ResponseMatch
    matched_checks: List[GuardMatch]
    all_checks: List[GuardMatch]



from acuvity import models
from acuvity.apexextend import ApexExtended
from acuvity.types import OptionalNullable, UNSET
from .httpclient import AsyncHttpClient, HttpClient
from .utils.logger import Logger
from .utils.retries import RetryConfig
from typing import Callable, Dict, Optional, Union
from .basesdk import BaseSDK
from .apexextend import ApexExtended

class Acuvity(BaseSDK):
    apex: ApexExtended

    # pylint: disable=super-init-not-called
    def __init__(
        self,
        security: Optional[
            Union[models.Security, Callable[[], models.Security]]
        ] = None,
        apex_domain: Optional[str] = None,
        apex_port: Optional[str] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        pass



# pylint: disable=protected-access

from typing import Callable, Dict, Optional, Union

import httpx

from acuvity import models
from acuvity.apexdiscovery import discover_apex
from acuvity.apexextend import ApexExtended
from acuvity.sdk import Acuvity
from acuvity.types import UNSET, OptionalNullable

from .httpclient import AsyncHttpClient, HttpClient
from .utils.logger import Logger
from .utils.retries import RetryConfig

# Save the original __init__ reference
__original_init__ = Acuvity.__init__

# Define the new __init__ method
def __patched_init__(
    self,
    security: Optional[
        Union[models.Security, Callable[[], models.Security]]
    ] = None,
    apex_domain: Optional[str] = None,
    apex_port: Optional[str] = None,
    server_idx: Optional[int] = None,
    server_url: Optional[str] = None,
    url_params: Optional[Dict[str, str]] = None,
    client: Optional[HttpClient] = None,
    async_client: Optional[AsyncHttpClient] = None,
    retry_config: OptionalNullable[RetryConfig] = UNSET,
    timeout_ms: Optional[int] = None,
    debug_logger: Optional[Logger] = None,
) -> None:
    r"""Instantiates the SDK configuring it with the provided parameters.

    :param security: The security details required for authentication
    :param apex_domain: Allows setting the apex_domain variable for url substitution
    :param apex_port: Allows setting the apex_port variable for url substitution
    :param server_idx: The index of the server to use for all methods
    :param server_url: The server URL to use for all methods
    :param url_params: Parameters to optionally template the server URL with
    :param client: The HTTP client to use for all synchronous methods
    :param async_client: The Async HTTP client to use for all asynchronous methods
    :param retry_config: The retry configuration to use for all supported methods
    :param timeout_ms: Optional request timeout applied to each operation in milliseconds
    """
    if client is None:
        client = httpx.Client()

    assert issubclass(
        type(client), HttpClient
    ), "The provided client must implement the HttpClient protocol."

    apex_domain, apex_port = discover_apex(
        client=client,
        server_url=server_url,
        security=security,
        apex_domain=apex_domain,
        apex_port=apex_port,
    )

    # Call the original __init__ using super
    __original_init__(
        self,
        security=security,
        apex_domain=apex_domain,
        apex_port=apex_port,
        server_idx=server_idx,
        server_url=server_url,
        url_params=url_params,
        client=client,
        async_client=async_client,
        retry_config=retry_config,
        timeout_ms=timeout_ms,
        debug_logger=debug_logger,
    )

# Define the new _init_sdks method
def __patched_init_sdks(self):
    self.apex = ApexExtended(self.sdk_configuration)

# Monkey-patch the __init__ and _init_sdks methods
setattr(Acuvity, "__init__", __patched_init__)
setattr(Acuvity, "_init_sdks", __patched_init_sdks)



from dataclasses import dataclass

from .constants import ComparisonOperator
from .errors import GuardConfigValidationError


@dataclass(frozen=False)
class Threshold:
    def __init__(self, threshold_str: str):
        """
        Parse threshold string into Threshold object.

        Args:
            threshold_str: Threshold string (e.g. '>= 0.8')

        Returns:
            Threshold object or None if parsing fails

        Raises:
            GuardConfigValidationError: If threshold format is invalid
        """
        try:
            # First try to convert the entire string to float (case when only number provided)
            try:
                self.value = float(threshold_str)
                if not 0 <= self.value <= 1:
                    raise GuardConfigValidationError("Invalid threshold value should be between 0-1")
                self.operator = ComparisonOperator.GREATER_EQUAL  # Default operator
                return
            except ValueError:
                pass

            # If that fails, try to split into operator and value
            parts = threshold_str.split()
            if len(parts) != 2:
                raise GuardConfigValidationError("Invalid threshold format")

            operator_str, value_str = parts
            self.value = float(value_str)
            if not 0 <= self.value <= 1:
                raise GuardConfigValidationError("Invalid threshold value should be between 0-1")

            try:
                self.operator = ComparisonOperator(operator_str)
            except ValueError as e:
                raise GuardConfigValidationError(f"Invalid operator: {operator_str}") from e

        except ValueError as e:
            raise GuardConfigValidationError("Invalid threshold format") from e

    def __str__(self) -> str:
        return f"{self.operator.value} {self.value}"

    def compare(self, value: float) -> bool:
        """
        Compare a value against a threshold.

        Args:
            threshold: Threshold object containing operator and value
            value: Value to compare against threshold

        Returns:
            True if value meets threshold criteria, False otherwise
        """
        if self.operator == ComparisonOperator.GREATER_THAN:
            return value > self.value
        if self.operator == ComparisonOperator.GREATER_EQUAL:
            return value >= self.value
        if self.operator == ComparisonOperator.EQUAL:
            return value == self.value
        if self.operator == ComparisonOperator.LESS_EQUAL:
            return value <= self.value
        if self.operator == ComparisonOperator.LESS_THAN:
            return value < self.value
        return False


DEFAULT_THRESHOLD = Threshold(">= 0.0")



# pylint: disable=protected-access

import base64
import os
from pathlib import Path
from typing import Dict, Iterable, List, Optional, Sequence, Union

from acuvity.guard.config import Guard, GuardConfig, GuardName
from acuvity.models import (
    Analyzer,
    Anonymization,
    Extractionrequest,
    Scanrequest,
    Type,
)
from acuvity.response.match import ScanResponseMatch
from acuvity.sdkconfiguration import SDKConfiguration
from acuvity.utils.logger import get_default_logger

from .apex import Apex

logger = get_default_logger()




class ApexExtended(Apex):
    def __init__(self, sdk_config: SDKConfiguration) -> None:
        super().__init__(sdk_config)
        self._available_analyzers: Optional[List[Analyzer]] = None

    def list_available_guards(self) -> List[str]:
        """
        list_available_guards: returns a list of all available guards that can be detected.
        """
        return GuardName.values()

    def list_detectable_secrets(self) -> List[str]:
        """
        list_detectable_secrets: returns a list of all available secrets that can be detected.
        """
        detectable_secrets: List[str] = []
        if self._available_analyzers is None:
            self._available_analyzers = self.list_analyzers()
        for analyzer in self._available_analyzers:
            if analyzer.detectors:
                detectable_secrets_local = [
                    str(detector.name)
                    for detector in analyzer.detectors
                    if detector.group == "Secrets"
                ]
                detectable_secrets.extend(detectable_secrets_local)
        return sorted(list(set(detectable_secrets)))

    def list_detectable_piis(self) -> List[str]:
        """
        list_detectable_piis: returns a list of all available Piis that can be detected.
        """
        detectable_piis: List[str] = []
        if self._available_analyzers is None:
            self._available_analyzers = self.list_analyzers()
        for analyzer in self._available_analyzers:
            if analyzer.detectors:
                detectable_piis_local = [
                    str(detector.name)
                    for detector in analyzer.detectors
                    if detector.group == "PIIs"
                ]
                detectable_piis.extend(detectable_piis_local)
        return sorted(list(set(detectable_piis)))

    def scan(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[Type,str] = Type.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        redactions: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        guard_config: Optional[Union[str, Path, Dict, List[Guard]]] = None,
    ) -> ScanResponseMatch:
        """
        scan() runs the provided messages (prompts) through the Acuvity detection engines and returns the results. Alternatively, you can run model output through the detection engines.
        Returns a Scanresponse object on success, and raises different exceptions on failure.

        This function allows to use and try different analyzers and make use of the redaction feature.
        You can also run access policies and content policies by passing them as parameters.

        :param messages: the messages to scan. These are the prompts that you want to scan. Required if no files or a direct request object are provided.
        :param files: the files to scan. These are the files that you want to scan. Required if no messages are provided. Can be used in addition to messages.
        :param request_type: the type of the validation. This can be either Type.INPUT or Type.OUTPUT. Defaults to Type.INPUT. Use Type.OUTPUT if you want to run model output through the detection engines.
        :param annotations: the annotations to use. These are the annotations that you want to use. If not provided, no annotations will be used.
        :param redactions: the redactions that need to be redacted if detected. This arg cannot be used with guard_config.
        :param keywords: the keywords that need to be detected. This arg cannot be used with guard_config.
        :param guard_config: the guard config used to do the response eval for matches. If not provided, the default guard config will be used.
        """

        raw_scan_response = self.scan_request(request=self.__build_scan_request(
            *messages,
            files=files,
            request_type=request_type,
            annotations=annotations,
            redactions=redactions,
            keywords=keywords,
            guard_config=guard_config if guard_config is None else GuardConfig(guard_config),
        ))

        # always send a guard config to the ScanResponseMatch
        try:
            if guard_config:
                gconfig = GuardConfig(guard_config)
            else:
                gconfig = GuardConfig()
        except Exception as e:
            logger.debug("Error while processing the guard config")
            raise ValueError("Cannot process the guard config") from e

        return ScanResponseMatch(raw_scan_response, gconfig, files=files)

    async def scan_async(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[Type,str] = Type.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        redactions: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        guard_config: Optional[Union[str, Path, Dict, List[Guard]]] = None,
    ) -> ScanResponseMatch:
        """
        scan_async() runs the provided messages (prompts) through the Acuvity detection engines and returns the results. Alternatively, you can run model output through the detection engines.
        Returns a Scanresponse object on success, and raises different exceptions on failure.

        This function allows to use and try different analyzers and make use of the redaction feature.
        You can also run access policies and content policies by passing them as parameters.

        :param messages: the messages to scan. These are the prompts that you want to scan. Required if no files or a direct request object are provided.
        :param files: the files to scan. These are the files that you want to scan. Required if no messages are provided. Can be used in addition to messages.        :param request_type: the type of the validation. This can be either Type.INPUT or Type.OUTPUT. Defaults to Type.INPUT. Use Type.OUTPUT if you want to run model output through the detection engines.
        :param annotations: the annotations to use. These are the annotations that you want to use. If not provided, no annotations will be used.
        :param analyzers: the analyzers to use. These are the analyzers that you want to use. If not provided, the internal default analyzers will be used. Use "+" to include an analyzer and "-" to exclude an analyzer. For example, ["+image-classifier", "-modality-detector"] will include the image classifier and exclude the modality detector. If any analyzer does not start with a '+' or '-', then the default analyzers will be replaced by whatever is provided. Call `list_analyzers()` and/or its variants to get a list of available analyzers.
        :param guard_config: the guard config used to do the response eval for matches. If not provided, the default guard config will be used.
        """
        raw_scan_response = await self.scan_request_async(request=self.__build_scan_request(
            *messages,
            files=files,
            request_type=request_type,
            annotations=annotations,
            redactions=redactions,
            keywords=keywords,
            guard_config=guard_config if guard_config is None else GuardConfig(guard_config),
        ))
        # always send a guard config to the ScanResponseMatch
        try:
            if guard_config:
                gconfig = GuardConfig(guard_config)
            else:
                gconfig = GuardConfig()
        except Exception as e:
            logger.debug("Error while processing the guard config")
            raise ValueError("Cannot process the guard config") from e

        return ScanResponseMatch(raw_scan_response, gconfig, files=files)

    def __build_scan_request(
        self,
        *messages: str,
        files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None,
        request_type: Union[Type,str] = Type.INPUT,
        annotations: Optional[Dict[str, str]] = None,
        redactions: Optional[List[str]] = None,
        keywords: Optional[List[str]] = None,
        anonymization: Union[Anonymization, str, None] = None,
        guard_config: Optional[GuardConfig] = None,
    ) -> Scanrequest:
        request = Scanrequest.model_construct()

        # if guard_config is given, the keywords and redactions args must not be given.
        if guard_config and (keywords or redactions):
            raise ValueError("Cannot specify keywords or redactions in scan args when using guard config. Please use only one.")

        # messages must be strings
        for message in messages:
            if not isinstance(message, str):
                raise ValueError(f"messages must be strings but was {type(message)}")
        if len(messages) == 0 and files is None:
            raise ValueError("no messages and no files provided")
        if len(messages) > 0:
            request.messages = list(messages)

        # files must be a list of strings (or paths) or a single string (or path)
        extractions: List[Extractionrequest] = []
        if files is not None:
            process_files: List[Union[os.PathLike, str]] = []
            if isinstance(files, str):
                process_files.append(files)
            elif isinstance(files, os.PathLike):
                process_files.append(files)
            elif isinstance(files, Iterable):
                for file in files:
                    if not isinstance(file, str) and not isinstance(file, os.PathLike):
                        raise ValueError("files must be strings or paths")
                    process_files.append(file)
            else:
                raise ValueError("files must be strings or paths")
            for process_file in process_files:
                with open(process_file, 'rb') as opened_file:
                    file_content = opened_file.read()
                    # base64 encode the file content and then append
                    extractions.append(Extractionrequest(
                        data=base64.b64encode(file_content).decode("utf-8"),
                    ))
        if len(extractions) > 0:
            request.extractions = extractions

        # request_type must be either "Input" or "Output"
        if isinstance(request_type, Type):
            request.type = request_type
        elif isinstance(request_type, str):
            if request_type not in ("Input", "Output"):
                raise ValueError("request_type must be either 'Input' or 'Output'")
            request.type = Type(request_type)
        else:
            raise ValueError("type must be a 'str' or 'Type'")

        # annotations must be a dictionary of strings
        if annotations is not None:
            if not isinstance(annotations, dict):
                raise ValueError("annotations must be a dictionary")
            for key, value in annotations.items():
                if not isinstance(key, str) or not isinstance(value, str):
                    raise ValueError("annotations must be strings")
            request.annotations = annotations

        # now here check the guard config and parse it for the redaction and keywords.
        if guard_config:
            keywords = guard_config.keywords
            redactions = guard_config.redaction_keys

        # anonymization must be "FixedSize" or "VariableSize"
        if anonymization is not None:
            if isinstance(anonymization, Anonymization):
                request.anonymization = anonymization
            elif isinstance(anonymization, str):
                if anonymization not in ("FixedSize", "VariableSize"):
                    raise ValueError("anonymization must be 'FixedSize' or 'VariableSize'")
                request.anonymization = Anonymization(anonymization)
            else:
                raise ValueError("anonymization must be a 'str' or 'Anonymization'")

        # redactions must be a list of strings
        if redactions is not None:
            if not isinstance(redactions, List):
                raise ValueError("redactions must be a list")
            for redaction in redactions:
                if not isinstance(redaction, str):
                    raise ValueError("redactions must be strings")
            request.redactions = redactions

        # keywords must be a list of strings
        if keywords is not None:
            if not isinstance(keywords, List):
                raise ValueError("keywords must be a list")
            for keyword in keywords:
                if not isinstance(keyword, str):
                    raise ValueError("keywords must be strings")
            request.keywords = keywords

        return request



from typing import Dict, List, Optional, Union

from acuvity.guard.config import Guard
from acuvity.guard.constants import GuardName
from acuvity.models.extraction import Extraction
from acuvity.models.textualdetection import Textualdetection, TextualdetectionType
from acuvity.response.result import GuardMatch, ResponseMatch
from acuvity.utils.logger import get_default_logger

logger = get_default_logger()

class ResponseHelper:
    """Parser for accessing values in Extraction response based on guard types."""

    @staticmethod
    def evaluate(
        extraction: Extraction,
        guard: Guard,
        match_name: Optional[str] = None
    ) -> GuardMatch:
        """
        Evaluates a check condition using a Guard object.

        Args:
            extraction: A type which provides all the extractions based on the detection engine.
            guard: A guard configuration
            match_name: The match match for the guard

        Returns:
            GuardMatch with MATCH.YES if condition met, MATCH>NO if not met
        """
        exists = False
        value = 0.0
        match_count = 0
        match_list: List[str] = []
        if guard.name in (GuardName.PROMPT_INJECTION, GuardName.JAILBREAK, GuardName.MALICIOUS_URL):
            exists, value = ResponseHelper._get_guard_value(extraction.exploits, str(guard.name))
        elif guard.name in (GuardName.TOXIC, GuardName.BIASED, GuardName.HARMFUL):
            exists, value = ResponseHelper._get_guard_value(extraction.malcontents, str(guard.name))
        elif guard.name == GuardName.LANGUAGE:
            # Language guard
            if match_name:
                exists, value = ResponseHelper._get_guard_value(extraction.languages, match_name)
            elif extraction.languages:
                exists, value = len(extraction.languages) > 0 , 1.0
        elif guard.name == GuardName.MODALITY:
            exists = ResponseHelper._get_modality_value(extraction, guard, match_name)
        elif guard.name == GuardName.PII_DETECTOR:
            exists, value, match_count, match_list = ResponseHelper._get_text_detections(extraction.pi_is, guard, TextualdetectionType.PII, extraction.detections, match_name)
        elif guard.name == GuardName.SECRETS_DETECTOR:
            exists, value, match_count, match_list = ResponseHelper._get_text_detections(extraction.secrets, guard, TextualdetectionType.SECRET, extraction.detections, match_name)
        elif guard.name == GuardName.KEYWORD_DETECTOR:
            exists, value, match_count, match_list = ResponseHelper._get_text_detections(extraction.keywords, guard, TextualdetectionType.KEYWORD, extraction.detections, match_name)

        # A match is found if the the detection exists value is greater than the threshold
        response_match=ResponseMatch.NO
        if exists and guard.threshold.compare(value):
            response_match=ResponseMatch.YES

        if response_match is ResponseMatch.NO:
            value = -1

        return GuardMatch(
            response_match=response_match,
            guard_name=guard.name,
            threshold=str(guard.threshold),
            actual_value=value,
            match_count=match_count,
            match_values=match_list
        )

    @staticmethod
    def _get_guard_value(
        lookup: Union[Dict[str, float] , None],
        prefix: str,
    ) -> tuple[bool, float]:
        if not lookup:
            return False, 0
        value = lookup.get(prefix)
        if value is not None:
            return True, float(value)
        return False, 0.0

    @staticmethod
    def _get_text_detections(
        lookup: Union[Dict[str, float] , None],
        guard: Guard,
        detection_type: TextualdetectionType,
        detections: Union[List[Textualdetection], None],
        match_name: Optional[str]
    )-> tuple[bool, float, int, List[str]]:

        if match_name:
            # Count occurrences in textual detections
            if not detections:
                return False, 0, 0, []
            text_matches = []
            text_matches = [
                d.score for d in detections
                if d.type == detection_type and d.name == match_name and d.score is not None  and guard.threshold.compare(d.score)
            ]

            count = len(text_matches)
            # If no textual detections, check `lookup` for the match
            if count == 0 and lookup and match_name in lookup:
                return True, lookup[match_name], 1, [match_name]

            if count == 0:
                return False, 0, 0, []

            score = max(text_matches)
            return True, score, count, [match_name]

        # Return all text match values if no match_name is provided
        exists = bool(lookup)
        count = len(lookup) if lookup else 0
        return exists, 1.0 if exists else 0.0, count, list(lookup.keys()) if lookup else []

    @staticmethod
    def _get_modality_value(
        extraction: Extraction,
        _: Guard,
        match_name: Optional[str] = None
    ) -> bool:
        if not extraction.modalities:
            return False  # No modalities at all

        if match_name:
            # Check for specific modality
            return any(m.group == match_name for m in extraction.modalities)

        # Check if any modality exists
        return len(extraction.modalities) > 0



"""Code originally generated by Speakeasy (https://speakeasy.com)."""

# pylint: disable=unused-import
import acuvity.apexextend

# pylint: disable=unused-import
import acuvity.sdkextend
from acuvity.guard import *
from acuvity.guard import Threshold
from acuvity.response import *

from ._version import __title__, __version__
from .models import *
from .sdk import *
from .sdkconfiguration import *

VERSION: str = __version__



from typing import Any, Callable, Optional, Tuple, Union
from urllib.parse import urlparse

import jwt
from pydantic import BaseModel

from acuvity import models
from acuvity.utils import get_security_from_env

from .httpclient import HttpClient


def discover_apex(
    client: HttpClient,
    server_url: Optional[str] = None,
    security: Optional[
        Union[models.Security, Callable[[], models.Security]]
    ] = None,
    apex_domain: Optional[str] = None,
    apex_port: Optional[str] = None,
) -> Tuple[Optional[str], Optional[str]]:
    # pylint: disable=too-many-return-statements
    """
    Discovers the apex domain and port from the encoded token or by calling the backend API if the server_url is not provided.
    This will raise exceptions if there is no token or the token is empty or invalid.
    """
    # if there is no token, then we can't perform discovery
    # however, a token is in general needed to perform any API calls as there are no unauthenticated endpoints
    # so we strictly check if there is a token and fail otherwise
    token: str = ""
    sec: Optional[BaseModel] = get_security_from_env(security, models.Security)
    if sec is None:
        raise ValueError("No security object provided, or ACUVITY_TOKEN environment variable is not set or empty")
    if not isinstance(sec, models.Security):
        raise ValueError("Security object is not of type Security")
    token = sec.token if sec.token is not None else sec.cookie if sec.cookie is not None else ""
    if token == "":
        raise ValueError("No token provided")

    # if a server_url was given, then we don't need to perform discovery at all
    # and we know already that we have a token, so we can return immediately
    if server_url is not None:
        return apex_domain, apex_port

    # if an apex_domain was given, then we also don't need to perform discovery
    # and we know already that we have a token, so we can return immediately
    if apex_domain is not None:
        return apex_domain, apex_port

    # decode the token but don't verify the signature
    try:
        decoded_token = jwt.decode(token, options={"verify_signature": False})
        if "iss" not in decoded_token:
            raise ValueError("token has no 'iss' field")
    except Exception as e:
        raise ValueError(f"invalid token: {e}") from e

    # check if the token has the apex URL encoded in the token
    token_apex_url: Optional[str] = None
    decoded_token_opaque: Optional[dict] = decoded_token.get("opaque", None)
    if decoded_token_opaque is not None:
        token_apex_url = decoded_token_opaque.get("apex-url", None)

    # if we have the Apex URL encoded in the token, then we parse and use that directly
    if token_apex_url is not None:
        if token_apex_url.startswith(("http://", "https://")):
            parsed_url = urlparse(token_apex_url)
            domain = parsed_url.hostname
            if parsed_url.port is not None:
                port = f"{parsed_url.port}"
            else:
                if parsed_url.scheme == "https":
                    port = "443"
                elif parsed_url.scheme == "http":
                    port = "80"
                else:
                    port = None
            if domain is None or domain == "":
                raise ValueError(f"JWT Apex URL has no domain: {token_apex_url}")
            if port is None or port == "":
                raise ValueError(f"JWT Apex URL has no port or wrong scheme: {token_apex_url}")
            return domain, port

        # we're assuming this must be a domain without a scheme
        return token_apex_url, "443"

    # otherwise we extract the API URL from the token
    api_url = decoded_token["iss"]
    if api_url == "":
        raise ValueError("'iss' field value of token is empty, but should have been the API URL")

    def well_known_apex_info(client: HttpClient, token: str, url: str, iteration: int = 0) -> Any:
        if iteration == 3:
            raise ValueError("Too many redirects")
        req = client.build_request("GET", url, headers={"Authorization": f"Bearer {token}"})
        resp = client.send(req, follow_redirects=False) # following redirects automatically will remove the token from the call as headers are not going to be sent anymore

        if resp.status_code == 401:
            raise ValueError("Unauthorized: Invalid token or insufficient permissions")

        if resp.is_redirect:
            return well_known_apex_info(client, token, resp.headers["Location"], iteration + 1)
        return resp.json()
    try:
        apex_info = well_known_apex_info(client, token, f"{api_url}/.well-known/acuvity/my-apex.json")
    except Exception as e:
        raise ValueError(f"Failed to get apex info from well-known endpoint: {str(e)}") from e

    try:
        # extract the information from the response
        apex_url: str = apex_info["url"]
        if apex_url == "":
            raise ValueError("Apex Info: no URL in response")
        port = f"{apex_info['portNoMTLS']}"
        if port == "":
            raise ValueError("Apex Info: no portNoMTLS in response")
        if apex_url.startswith(("http://", "https://")):
            # parse the URL to extract the domain
            # use hostname as opposed to netloc because we *only* want the domain, and not the domain:port notation
            parsed_url = urlparse(apex_url)
            domain = parsed_url.hostname
        else:
            domain = apex_url
        if domain == "":
            raise ValueError(f"Apex Info: no domain in URL: f{apex_url}")
    except Exception as e:
        raise ValueError("Failed to extract apex info from response") from e

    return domain, port



class ValidationError(Exception):
    """Raised when validation fails."""

class GuardConfigError(Exception):
    """Base exception for guard parser errors"""

class GuardConfigValidationError(GuardConfigError):
    """Raised when config validation fails"""

class GuardThresholdParsingError(GuardConfigError):
    """Raised when threshold parsing fails"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from acuvity.types import BaseModel
from enum import Enum
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TextualdetectionType(str, Enum):
    r"""The type of detection."""

    KEYWORD = "Keyword"
    PII = "PII"
    SECRET = "Secret"
    CDT = "CDT"


class TextualdetectionTypedDict(TypedDict):
    r"""Represents a textual detection done by policy."""

    end: NotRequired[int]
    r"""The end position of the detection in the original data."""
    key: NotRequired[str]
    r"""The key that is used in the name's place, If empty, a sequence of X's are used."""
    name: NotRequired[str]
    r"""The name of the detection."""
    redacted: NotRequired[bool]
    r"""If true this detection has been redacted."""
    redacted_end: NotRequired[int]
    r"""The end position of the detection in the redacted data."""
    redacted_start: NotRequired[int]
    r"""The start position of the detection in the redacted data."""
    score: NotRequired[float]
    r"""The confidence score of the detection."""
    start: NotRequired[int]
    r"""The start position of the detection in the original data."""
    type: NotRequired[TextualdetectionType]
    r"""The type of detection."""


class Textualdetection(BaseModel):
    r"""Represents a textual detection done by policy."""

    end: Optional[int] = None
    r"""The end position of the detection in the original data."""

    key: Optional[str] = None
    r"""The key that is used in the name's place, If empty, a sequence of X's are used."""

    name: Optional[str] = None
    r"""The name of the detection."""

    redacted: Optional[bool] = None
    r"""If true this detection has been redacted."""

    redacted_end: Annotated[Optional[int], pydantic.Field(alias="redactedEnd")] = None
    r"""The end position of the detection in the redacted data."""

    redacted_start: Annotated[Optional[int], pydantic.Field(alias="redactedStart")] = (
        None
    )
    r"""The start position of the detection in the redacted data."""

    score: Optional[float] = None
    r"""The confidence score of the detection."""

    start: Optional[int] = None
    r"""The start position of the detection in the original data."""

    type: Optional[TextualdetectionType] = None
    r"""The type of detection."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from acuvity.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PrincipalappTypedDict(TypedDict):
    r"""Describes the principal information of an application."""

    component: NotRequired[str]
    r"""The component of the application request."""
    labels: NotRequired[List[str]]
    r"""The list of labels attached to an application request."""
    name: NotRequired[str]
    r"""The name of the application."""


class Principalapp(BaseModel):
    r"""Describes the principal information of an application."""

    component: Optional[str] = None
    r"""The component of the application request."""

    labels: Optional[List[str]] = None
    r"""The list of labels attached to an application request."""

    name: Optional[str] = None
    r"""The name of the application."""



import os
from typing import List, Sequence, Union

from acuvity.guard.config import GuardConfig
from acuvity.guard.constants import GuardName
from acuvity.models.scanresponse import Scanresponse
from acuvity.response.processor import ResponseProcessor
from acuvity.response.result import GuardMatch, Matches, ResponseMatch


class ScanResponseMatch:
    """
    Wrapper for Scanresponse to add functionality for checking guards.
    """
    def __init__(self, scan_response: Scanresponse, guard_config: GuardConfig,
                files: Union[Sequence[Union[str,os.PathLike]], os.PathLike, str, None] = None):
        self._guard_config = guard_config
        self.scan_response = scan_response
        self._number_of_files = self._count_files(files)
        if self._guard_config is None:
            raise ValueError("No guard configuration was passed or available in the instance.")

        # compute the match
        try:
            self.match_details = ResponseProcessor(self.scan_response, self._guard_config).matches()
        except Exception as e:
            raise ValueError(f"Failed to process match: {str(e)}") from e

    def _count_files(self, files: Union[Sequence[Union[str, os.PathLike]], os.PathLike, str, None] = None) -> int:
        if files is None:
            return 0
        return len([files] if isinstance(files, (str, os.PathLike)) else files)

    def matches(self, file_index: int = -1, msg_index: int = -1) -> List[Matches]:
        """
        Returns the overall match of the scan response.
        """
        matches: List[Matches] = []
        # Helper to search one index
        def search_at_index(idx: int) -> Matches:
            if 0 <= idx < len(self.match_details):
                return self.match_details[idx]
            raise ValueError(f"Invalid index {idx}")

        # 1) If either index is given (not -1), search them
        if file_index != -1 or msg_index != -1:
            if file_index != -1:
                matches.append(search_at_index(file_index))
            if msg_index != -1:
                idx_msg = msg_index + self._number_of_files
                matches.append(search_at_index(idx_msg))
            return matches

        # 2) If both are -1, return all
        return self.match_details

    def guard_match(self, guard: GuardName, file_index: int = -1, msg_index: int = -1) -> List[GuardMatch]:
        """
        Retrieves a single guard's match.

        Args:
            guard: Name of the guard for querying a specific guard's match.
            file_index: guard check to run against which input file.
            msg_index: guard check to run aginst which input msg.

        Returns:
            The 1st found GuardMatch for a specific guard.
        """
        matches: List[GuardMatch] = []
        if not GuardName.valid(str(guard)):
            raise ValueError(f"Invalid gaurd name, please provide one of the following {GuardName.values()}")

        # Helper to search one index
        def search_at_index(idx: int):
            if 0 <= idx < len(self.match_details):
                for c in self.match_details[idx].matched_checks:
                    if c.guard_name == guard:
                        matches.append(c)
            else:
                raise ValueError(f"Index {idx} is out of range.")

        # 1) If either index is given (not -1), search them
        if file_index != -1 or msg_index != -1:
            if file_index != -1:
                search_at_index(file_index)
            if msg_index != -1:
                idx_msg = msg_index + self._number_of_files
                search_at_index(idx_msg)
        else:
            # 2) If both are -1, search all
            for check in self.match_details:
                for c in check.matched_checks:
                    if c.guard_name == guard:
                        matches.append(c)

        # 3) If no matches found, return a "default" match
        if not matches:
            matches.append(GuardMatch(
                response_match=ResponseMatch.NO,
                guard_name=guard,
                threshold="> 0.0",
                actual_value=0.0
            ))

        return matches

    def __getattr__(self, name):
        """
        Delegate attribute access to the original Scanresponse object.
        """
        return getattr(self.scan_response, name)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .extractionrequest import Extractionrequest, ExtractionrequestTypedDict
from acuvity.types import BaseModel
from enum import Enum
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Anonymization(str, Enum):
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""

    FIXED_SIZE = "FixedSize"
    VARIABLE_SIZE = "VariableSize"


class Type(str, Enum):
    r"""The type of text."""

    INPUT = "Input"
    OUTPUT = "Output"


class ScanrequestTypedDict(TypedDict):
    r"""This is a scan request."""

    access_policy: NotRequired[str]
    r"""AccessPolicy allows to pass optional Rego access policy. If not set,
    The action is always Allow,
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """
    analyzers: NotRequired[List[str]]
    r"""The analyzers parameter allows for customizing which analyzers should be used,
    overriding the default selection. Each analyzer entry can optionally include a
    prefix to modify its behavior:

    - No prefix: Runs only the specified analyzers and any dependencies required
    for deeper analyzis (slower but more acurate).
    - '+' (enable): Activates an analyzer that is disabled by default.
    - '-' (disable): Disables an analyzer that is enabled by default.
    - '@' (direct execution): Runs the analyzer immediately, bypassing the deeper
    analyzis (faster but less acurate).

    An analyzers entry can be specified using:
    - The analyzer name (e.g., 'Toxicity detector')
    - The analyzer ID (e.g., 'en-text-toxicity-detector')
    - The analyzer group (e.g., 'Detectors')
    - A detector name (e.g., 'toxic')
    - A detector label (e.g., 'insult')
    - A detector group (e.g., 'Malcontents')

    If left empty, all default analyzers will be executed.
    """
    annotations: NotRequired[Dict[str, str]]
    r"""Annotations attached to the extraction."""
    anonymization: NotRequired[Anonymization]
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""
    bypass_hash: NotRequired[str]
    r"""In the case of a contentPolicy that asks for a confirmation, this is the
    hash you must send back to bypass the block. This is only useful when a
    content policy has been set or is evaluated remotely.
    """
    content_policy: NotRequired[str]
    r"""ContentPolicy allows to pass optional Rego content policy. If not set,
    The action is always Allow, and there cannot be any alerts raised etc
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """
    extractions: NotRequired[List[ExtractionrequestTypedDict]]
    r"""The extractions to request."""
    keywords: NotRequired[List[str]]
    r"""The keywords found during classification."""
    messages: NotRequired[List[str]]
    r"""Messages to process and provide detections for. Use data in extractions for
    processing binary data.
    """
    minimal_logging: NotRequired[bool]
    r"""If true, the system will not log the contents that were scanned."""
    redactions: NotRequired[List[str]]
    r"""The redactions to perform if they are detected."""
    type: NotRequired[Type]
    r"""The type of text."""


class Scanrequest(BaseModel):
    r"""This is a scan request."""

    access_policy: Annotated[Optional[str], pydantic.Field(alias="accessPolicy")] = None
    r"""AccessPolicy allows to pass optional Rego access policy. If not set,
    The action is always Allow,
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """

    analyzers: Optional[List[str]] = None
    r"""The analyzers parameter allows for customizing which analyzers should be used,
    overriding the default selection. Each analyzer entry can optionally include a
    prefix to modify its behavior:

    - No prefix: Runs only the specified analyzers and any dependencies required
    for deeper analyzis (slower but more acurate).
    - '+' (enable): Activates an analyzer that is disabled by default.
    - '-' (disable): Disables an analyzer that is enabled by default.
    - '@' (direct execution): Runs the analyzer immediately, bypassing the deeper
    analyzis (faster but less acurate).

    An analyzers entry can be specified using:
    - The analyzer name (e.g., 'Toxicity detector')
    - The analyzer ID (e.g., 'en-text-toxicity-detector')
    - The analyzer group (e.g., 'Detectors')
    - A detector name (e.g., 'toxic')
    - A detector label (e.g., 'insult')
    - A detector group (e.g., 'Malcontents')

    If left empty, all default analyzers will be executed.
    """

    annotations: Optional[Dict[str, str]] = None
    r"""Annotations attached to the extraction."""

    anonymization: Optional[Anonymization] = Anonymization.FIXED_SIZE
    r"""How to anonymize the data. If deanonymize is true, then VariablSize is required."""

    bypass_hash: Annotated[Optional[str], pydantic.Field(alias="bypassHash")] = None
    r"""In the case of a contentPolicy that asks for a confirmation, this is the
    hash you must send back to bypass the block. This is only useful when a
    content policy has been set or is evaluated remotely.
    """

    content_policy: Annotated[Optional[str], pydantic.Field(alias="contentPolicy")] = (
        None
    )
    r"""ContentPolicy allows to pass optional Rego content policy. If not set,
    The action is always Allow, and there cannot be any alerts raised etc
    If it is set, it will be run, and the final decision will be computed based
    on that policy.
    If the rego code does not start with package main, then the needed
    classic package definition and  acuvity imports will be added
    automatically.
    If the code starts with package main, then everything remains untouched.
    """

    extractions: Optional[List[Extractionrequest]] = None
    r"""The extractions to request."""

    keywords: Optional[List[str]] = None
    r"""The keywords found during classification."""

    messages: Optional[List[str]] = None
    r"""Messages to process and provide detections for. Use data in extractions for
    processing binary data.
    """

    minimal_logging: Annotated[
        Optional[bool], pydantic.Field(alias="minimalLogging")
    ] = None
    r"""If true, the system will not log the contents that were scanned."""

    redactions: Optional[List[str]] = None
    r"""The redactions to perform if they are detected."""

    type: Optional[Type] = None
    r"""The type of text."""



from .config import Guard, GuardConfig, Match
from .constants import GuardName
from .threshold import Threshold

__all__ = [
    'Guard',
    'GuardConfig',
    'GuardName',
    'Threshold',
    'Match'
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from acuvity.types import BaseModel
import pydantic
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ExtractionrequestTypedDict(TypedDict):
    r"""Represents the extraction that the user wants to extract."""

    annotations: NotRequired[Dict[str, str]]
    r"""Annotations attached to the extraction."""
    data: NotRequired[str]
    r"""The binary data to request extraction for."""
    internal: NotRequired[bool]
    r"""If true, this extraction is for internal use only. This can be used by agentic
    systems to mark an extraction as internal only as opposed to user facing.
    """
    is_file: NotRequired[bool]
    r"""If true, the data of the extraction is a file."""
    is_stored: NotRequired[bool]
    r"""If true, indicates that the file has been stored."""
    label: NotRequired[str]
    r"""A means of distinguishing what was extracted, such as prompt, input file or
    code.
    """


class Extractionrequest(BaseModel):
    r"""Represents the extraction that the user wants to extract."""

    annotations: Optional[Dict[str, str]] = None
    r"""Annotations attached to the extraction."""

    data: Optional[str] = None
    r"""The binary data to request extraction for."""

    internal: Optional[bool] = None
    r"""If true, this extraction is for internal use only. This can be used by agentic
    systems to mark an extraction as internal only as opposed to user facing.
    """

    is_file: Annotated[Optional[bool], pydantic.Field(alias="isFile")] = None
    r"""If true, the data of the extraction is a file."""

    is_stored: Annotated[Optional[bool], pydantic.Field(alias="isStored")] = None
    r"""If true, indicates that the file has been stored."""

    label: Optional[str] = None
    r"""A means of distinguishing what was extracted, such as prompt, input file or
    code.
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .principalapp import Principalapp, PrincipalappTypedDict
from .principaluser import Principaluser, PrincipaluserTypedDict
from acuvity.types import BaseModel
from enum import Enum
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AuthType(str, Enum):
    r"""The type of authentication."""

    CERTIFICATE = "Certificate"
    USER_TOKEN = "UserToken"
    APP_TOKEN = "AppToken"
    HOSTNAME = "Hostname"
    TOKEN = "Token"
    COMPONENT_TOKEN = "ComponentToken"
    EXTERNAL = "External"


class PrincipalType(str, Enum):
    r"""The type of principal."""

    USER = "User"
    APP = "App"


class PrincipalTypedDict(TypedDict):
    r"""Describe the principal."""

    type: PrincipalType
    r"""The type of principal."""
    app: NotRequired[PrincipalappTypedDict]
    r"""Describes the principal information of an application."""
    auth_type: NotRequired[AuthType]
    r"""The type of authentication."""
    claims: NotRequired[List[str]]
    r"""List of claims extracted from the user query."""
    team: NotRequired[str]
    r"""The team that was used to authorize the request."""
    token_name: NotRequired[str]
    r"""The name of the token, if any."""
    user: NotRequired[PrincipaluserTypedDict]
    r"""Describes the principal information of a user."""


class Principal(BaseModel):
    r"""Describe the principal."""

    type: PrincipalType
    r"""The type of principal."""

    app: Optional[Principalapp] = None
    r"""Describes the principal information of an application."""

    auth_type: Annotated[Optional[AuthType], pydantic.Field(alias="authType")] = None
    r"""The type of authentication."""

    claims: Optional[List[str]] = None
    r"""List of claims extracted from the user query."""

    team: Optional[str] = None
    r"""The team that was used to authorize the request."""

    token_name: Annotated[Optional[str], pydantic.Field(alias="tokenName")] = None
    r"""The name of the token, if any."""

    user: Optional[Principaluser] = None
    r"""Describes the principal information of a user."""



from typing import List

from acuvity.guard.config import Guard, GuardConfig
from acuvity.models.scanresponse import Extraction, Scanresponse
from acuvity.response.helper import ResponseHelper
from acuvity.response.result import GuardMatch, Matches, ResponseMatch
from acuvity.utils.logger import get_default_logger

logger = get_default_logger()

class ResponseProcessor:
    """Handles processing of guard configurations."""

    def __init__(self, response: Scanresponse, guard_config: GuardConfig):
        self.guard_config = guard_config
        self._response = response

    def _process_guard(self, guard: Guard, extraction: Extraction) -> GuardMatch:
        """Process a guard with matches."""
        if guard.matches is None or len(guard.matches) == 0:
            return ResponseHelper.evaluate(extraction, guard)

        # Process guards with matchedqq
        result_match = ResponseMatch.NO
        match_counter = 0
        match_list : List[str] = []
        for match_name, match_name_guard in guard.matches.items():
            result = ResponseHelper.evaluate(extraction, guard, match_name)
            # increment the match_counter only if eval is YES and it crosess the individual count_threshold .
            if result.response_match == ResponseMatch.YES and result.match_count >= match_name_guard.count_threshold:
                match_counter += result.match_count
                match_list.append(match_name)
                # if any one match, then flagged or if threshold given then check if greater.
                if match_counter >= guard.count_threshold:
                    result_match = ResponseMatch.YES

        logger.debug("match guard {%s} , check {%s}, total match {%s}, guard threshold {%s}, match_list {%s}",
                    guard.name, result_match, match_counter, guard.count_threshold, match_list)

        # reset the match_list
        if result_match == ResponseMatch.NO:
            match_list = []
        return GuardMatch(
                    response_match=result_match,
                    guard_name=guard.name,
                    threshold=str(guard.threshold),
                    actual_value=1.0,
                    match_count=match_counter,
                    match_values=match_list
                )

    def matches(self) -> List[Matches]:
        """Process the complete guard configuration."""
        all_matches : List[Matches] = []
        try:
            if self._response.extractions is None:
                raise ValueError("response doesn't contain extractions")

            for ext in self._response.extractions:
                if ext.data is None:
                    continue
                matched_checks = []
                all_checks = []
                for guard in self.guard_config.guards:
                    result = self._process_guard(guard, ext)
                    if result.response_match == ResponseMatch.YES:
                        matched_checks.append(result)
                    all_checks.append(result)

                single_match =  Matches(
                    input_data=ext.data,
                    response_match=ResponseMatch.YES if matched_checks else ResponseMatch.NO,
                    matched_checks=matched_checks,
                    all_checks=all_checks
                )
                all_matches.append(single_match)
            return all_matches

        except Exception as e:
            raise ValueError(f"Failed to process guard configuration: {str(e)}") from e



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from acuvity.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LatencyTypedDict(TypedDict):
    r"""Holds information about latencies introduced by Apex."""

    access_policy: NotRequired[int]
    r"""How much time it took to run the access policy in nanoseconds."""
    analysis: NotRequired[int]
    r"""How much time it took to run content analysis in nanoseconds."""
    assign_policy: NotRequired[int]
    r"""How much time it took to run the assign policy in nanoseconds."""
    content_policy: NotRequired[int]
    r"""How much time it took to run content policy in nanoseconds."""
    extraction: NotRequired[int]
    r"""How much time it took to run input or output extraction in nanoseconds."""


class Latency(BaseModel):
    r"""Holds information about latencies introduced by Apex."""

    access_policy: Annotated[Optional[int], pydantic.Field(alias="accessPolicy")] = None
    r"""How much time it took to run the access policy in nanoseconds."""

    analysis: Optional[int] = None
    r"""How much time it took to run content analysis in nanoseconds."""

    assign_policy: Annotated[Optional[int], pydantic.Field(alias="assignPolicy")] = None
    r"""How much time it took to run the assign policy in nanoseconds."""

    content_policy: Annotated[Optional[int], pydantic.Field(alias="contentPolicy")] = (
        None
    )
    r"""How much time it took to run content policy in nanoseconds."""

    extraction: Optional[int] = None
    r"""How much time it took to run input or output extraction in nanoseconds."""



from enum import Enum
from typing import List, Optional


class ComparisonOperator(Enum):
    """Valid comparison operators for thresholds"""
    GREATER_THAN = '>'
    GREATER_EQUAL = '>='
    EQUAL = '=='
    LESS_EQUAL = '<='
    LESS_THAN = '<'

class GuardName(Enum):
    PROMPT_INJECTION = "prompt_injection"
    JAILBREAK = "jailbreak"
    MALICIOUS_URL = "malicious_url"
    TOXIC = "toxic"
    BIASED = "biased"
    HARMFUL = "harmful"
    LANGUAGE = "language"
    MODALITY = "modality"
    PII_DETECTOR = "pii_detector"
    SECRETS_DETECTOR = "secrets_detector"
    KEYWORD_DETECTOR = "keyword_detector"

    def __str__(self) -> str:
        """
        Return the string representation of the enum member (i.e., its value).
        """
        return self.value

    @classmethod
    def values(cls) -> List[str]:
        return [member.value for member in cls]

    @classmethod
    def valid(cls, guard: str) -> bool:
        """
        Check if the input string represents a valid guard name.

        Args:
            input: Input string to check.

        Returns:
            bool: True if the input matches a guard name, otherwise False.
        """
        # Check against GuardName enum values
        if guard in {guard.value for guard in GuardName}:
            return True
        return False

    @classmethod
    def get(cls, name: str) -> Optional['GuardName']:
        try:
            return cls(name.lower())
        except ValueError:
            return None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from abc import ABC, abstractmethod
from acuvity.httpclient import HttpClient
import httpx
from typing import Any, Callable, List, Optional, Tuple, Union


class HookContext:
    base_url: str
    operation_id: str
    oauth2_scopes: Optional[List[str]] = None
    security_source: Optional[Union[Any, Callable[[], Any]]] = None

    def __init__(
        self,
        base_url: str,
        operation_id: str,
        oauth2_scopes: Optional[List[str]],
        security_source: Optional[Union[Any, Callable[[], Any]]],
    ):
        self.base_url = base_url
        self.operation_id = operation_id
        self.oauth2_scopes = oauth2_scopes
        self.security_source = security_source


class BeforeRequestContext(HookContext):
    def __init__(self, hook_ctx: HookContext):
        super().__init__(
            hook_ctx.base_url,
            hook_ctx.operation_id,
            hook_ctx.oauth2_scopes,
            hook_ctx.security_source,
        )


class AfterSuccessContext(HookContext):
    def __init__(self, hook_ctx: HookContext):
        super().__init__(
            hook_ctx.base_url,
            hook_ctx.operation_id,
            hook_ctx.oauth2_scopes,
            hook_ctx.security_source,
        )


class AfterErrorContext(HookContext):
    def __init__(self, hook_ctx: HookContext):
        super().__init__(
            hook_ctx.base_url,
            hook_ctx.operation_id,
            hook_ctx.oauth2_scopes,
            hook_ctx.security_source,
        )


class SDKInitHook(ABC):
    @abstractmethod
    def sdk_init(self, base_url: str, client: HttpClient) -> Tuple[str, HttpClient]:
        pass


class BeforeRequestHook(ABC):
    @abstractmethod
    def before_request(
        self, hook_ctx: BeforeRequestContext, request: httpx.Request
    ) -> Union[httpx.Request, Exception]:
        pass


class AfterSuccessHook(ABC):
    @abstractmethod
    def after_success(
        self, hook_ctx: AfterSuccessContext, response: httpx.Response
    ) -> Union[httpx.Response, Exception]:
        pass


class AfterErrorHook(ABC):
    @abstractmethod
    def after_error(
        self,
        hook_ctx: AfterErrorContext,
        response: Optional[httpx.Response],
        error: Optional[Exception],
    ) -> Union[Tuple[Optional[httpx.Response], Optional[Exception]], Exception]:
        pass


class Hooks(ABC):
    @abstractmethod
    def register_sdk_init_hook(self, hook: SDKInitHook):
        pass

    @abstractmethod
    def register_before_request_hook(self, hook: BeforeRequestHook):
        pass

    @abstractmethod
    def register_after_success_hook(self, hook: AfterSuccessHook):
        pass

    @abstractmethod
    def register_after_error_hook(self, hook: AfterErrorHook):
        pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .modality import Modality, ModalityTypedDict
from .textualdetection import Textualdetection, TextualdetectionTypedDict
from acuvity.types import BaseModel
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ExtractionTypedDict(TypedDict):
    r"""Represents the extracted information to log."""

    pi_is: NotRequired[Dict[str, float]]
    r"""The PIIs found during classification.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'PIIs' accross all analyzers.

    Example of PIIs detected: aba_routing_number, address, bank_account,
    bitcoin_wallet, credit_card, driver_license, email_address, itin_number,
    location, medical_license, money_amount, passport_number, person, phone_number,
    ssn.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    annotations: NotRequired[Dict[str, str]]
    r"""Annotations attached to the extraction."""
    categories: NotRequired[List[ModalityTypedDict]]
    r"""The categories are remapping of the modalities in a more human friendly way."""
    confidentiality: NotRequired[float]
    r"""The level of general confidentiality of the input."""
    custom_data_types: NotRequired[Dict[str, float]]
    r"""The custom data types found during classification.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    data: NotRequired[str]
    r"""The data extracted."""
    detections: NotRequired[List[TextualdetectionTypedDict]]
    r"""The textual detections found while applying policies."""
    exploits: NotRequired[Dict[str, float]]
    r"""The various exploits attempts.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Exploits' accross all analyzers.

    Example of exploits detected: jailbreak, prompt_injection, malicious_url.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    hash: NotRequired[str]
    r"""The hash of the extraction."""
    intent: NotRequired[Dict[str, float]]
    r"""The estimated intent embodied into the text.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Intent' accross all analyzers.

    Example of intent detected: explain, outline, summarize, write.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    internal: NotRequired[bool]
    r"""If true, this extraction is for internal use only. This can be used by agentic
    systems to mark an extraction as internal only as opposed to user facing.
    """
    is_file: NotRequired[bool]
    r"""If true, the data of the extraction is a file."""
    is_stored: NotRequired[bool]
    r"""If true, indicates that the file has been stored."""
    keywords: NotRequired[Dict[str, float]]
    r"""The keywords found during classification.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    label: NotRequired[str]
    r"""A means of distinguishing what was extracted, such as prompt, input file or
    code.
    """
    languages: NotRequired[Dict[str, float]]
    r"""The language of the classification.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Languages' accross all analyzers.

    Example of languages: chinese, english, french, german, gibberish, japanese,
    russian, spanish.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    malcontents: NotRequired[Dict[str, float]]
    r"""The various malcontents attempts.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Malcontents' accross all analyzers.

    Example of malcontents: biased, harmful, toxic.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    modalities: NotRequired[List[ModalityTypedDict]]
    r"""The modalities of data detected in the data."""
    relevance: NotRequired[float]
    r"""The level of general organization relevance of the input.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    secrets: NotRequired[Dict[str, float]]
    r"""The secrets found during classification.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Secrets' accross all analyzers.

    Example of secrets: adafruit, alibaba, anthropic, apideck, apify, atlassian,
    aws_secret_key, buildkite, checkout, clickuppersonal, contentfulpersonalaccess,
    credentials, database_url_with_credentials, databricks, denodeploy, dfuse,
    digitalocean, discord_webhook, docker_hub, doppler, dropbox, endorlabs,
    fleetbase, flutterwave, frameio, freshdesk, fullstory, github, gitlab,
    gocardless, google_api, grafana, groq, huggingface, intra42, jwt, klaviyo,
    launchdarkly, linearapi, locationiq, mailchimp, mailgun, mapbox, maxmind,
    microsoft_teams_webhook, nightfall, notion, npm, openai, otp_auth_url, pagarme,
    paystack, planetscale, planetscaledb, portainer, posthog, postman, prefect,
    private_key, pubnub_publish, pubnub_subscribe, pulumi, ramp, razorpay, readme,
    rechargepayments, replicate, rubygems, salesforce, sendgrid, sendinblue,
    shopify, slack_access, slack_bot, slack_refresh, slack_user, slack_webhook,
    slack_workflow_webhook, sourcegraph, sourcegraphcody, squareapp, squareup,
    stripe, supabase, tailscale, tines_webhook, trufflehog, twilio, ubidots,
    voiceflow, web_url_with_credentials, zapierwebhook.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
    topics: NotRequired[Dict[str, float]]
    r"""The topic of the classification.

    The current list can be obtained through the analyzers API, with the following
    conditions:
    1. Includes all analyzers EXCEPT those in the 'Modality' group
    2. Includes all analyzer detectors EXCEPT those in these groups:
    - 'Secrets'
    - 'Exploits'
    - 'Intent'
    - 'PIIs'
    - 'Malcontents'

    Example of topics: category/enterprise, category/healthcare, category/medical,
    category/personal, contains/blank_image, contains/certificate_signing_request,
    contains/database_url, contains/handwritten_text, contains/printed_text,
    contains/public_key, contains/ssh_public_key, contains/web_url,
    decoded/base32_data, decoded/base58_data, decoded/base64_data,
    decoded/base64_url_data, decoded/base85_data, decoded/hexadecimal_data,
    decoded/puny_code_data, decoded/quoted_printable_data,
    decoded/url_encoding_data, department/compliance, department/cybersecurity,
    department/finance, department/human_resources,
    department/information_technology, department/legal, department/logistics,
    department/marketing, department/performance_evaluation,
    department/product_management, department/project_management, department/sales,
    department/strategy, department/support, depict/check, depict/code,
    depict/document, depict/unclassified, depict/whiteboard, domain/commercial,
    domain/development, domain/financial, domain/general, domain/regulatory,
    domain/workforce, extracted/csv_cell_content, extracted/document_text_content,
    extracted/handwritten_text_content, extracted/pdf_text_content,
    extracted/pptx_text_content, extracted/spreadsheet_text_content,
    extracted/typed_text_content, timeframe/current_year, timeframe/future,
    timeframe/last_year, timeframe/next_year, timeframe/past,
    transcribed/audio_text_content.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """


class Extraction(BaseModel):
    r"""Represents the extracted information to log."""

    pi_is: Annotated[Optional[Dict[str, float]], pydantic.Field(alias="PIIs")] = None
    r"""The PIIs found during classification.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'PIIs' accross all analyzers.

    Example of PIIs detected: aba_routing_number, address, bank_account,
    bitcoin_wallet, credit_card, driver_license, email_address, itin_number,
    location, medical_license, money_amount, passport_number, person, phone_number,
    ssn.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    annotations: Optional[Dict[str, str]] = None
    r"""Annotations attached to the extraction."""

    categories: Optional[List[Modality]] = None
    r"""The categories are remapping of the modalities in a more human friendly way."""

    confidentiality: Optional[float] = None
    r"""The level of general confidentiality of the input."""

    custom_data_types: Annotated[
        Optional[Dict[str, float]], pydantic.Field(alias="customDataTypes")
    ] = None
    r"""The custom data types found during classification.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    data: Optional[str] = None
    r"""The data extracted."""

    detections: Optional[List[Textualdetection]] = None
    r"""The textual detections found while applying policies."""

    exploits: Optional[Dict[str, float]] = None
    r"""The various exploits attempts.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Exploits' accross all analyzers.

    Example of exploits detected: jailbreak, prompt_injection, malicious_url.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    hash: Optional[str] = None
    r"""The hash of the extraction."""

    intent: Optional[Dict[str, float]] = None
    r"""The estimated intent embodied into the text.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Intent' accross all analyzers.

    Example of intent detected: explain, outline, summarize, write.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    internal: Optional[bool] = None
    r"""If true, this extraction is for internal use only. This can be used by agentic
    systems to mark an extraction as internal only as opposed to user facing.
    """

    is_file: Annotated[Optional[bool], pydantic.Field(alias="isFile")] = None
    r"""If true, the data of the extraction is a file."""

    is_stored: Annotated[Optional[bool], pydantic.Field(alias="isStored")] = None
    r"""If true, indicates that the file has been stored."""

    keywords: Optional[Dict[str, float]] = None
    r"""The keywords found during classification.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    label: Optional[str] = None
    r"""A means of distinguishing what was extracted, such as prompt, input file or
    code.
    """

    languages: Optional[Dict[str, float]] = None
    r"""The language of the classification.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Languages' accross all analyzers.

    Example of languages: chinese, english, french, german, gibberish, japanese,
    russian, spanish.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    malcontents: Optional[Dict[str, float]] = None
    r"""The various malcontents attempts.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Malcontents' accross all analyzers.

    Example of malcontents: biased, harmful, toxic.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    modalities: Optional[List[Modality]] = None
    r"""The modalities of data detected in the data."""

    relevance: Optional[float] = None
    r"""The level of general organization relevance of the input.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    secrets: Optional[Dict[str, float]] = None
    r"""The secrets found during classification.

    The current list can be obtained through the analyzers API by searching for
    detector groups 'Secrets' accross all analyzers.

    Example of secrets: adafruit, alibaba, anthropic, apideck, apify, atlassian,
    aws_secret_key, buildkite, checkout, clickuppersonal, contentfulpersonalaccess,
    credentials, database_url_with_credentials, databricks, denodeploy, dfuse,
    digitalocean, discord_webhook, docker_hub, doppler, dropbox, endorlabs,
    fleetbase, flutterwave, frameio, freshdesk, fullstory, github, gitlab,
    gocardless, google_api, grafana, groq, huggingface, intra42, jwt, klaviyo,
    launchdarkly, linearapi, locationiq, mailchimp, mailgun, mapbox, maxmind,
    microsoft_teams_webhook, nightfall, notion, npm, openai, otp_auth_url, pagarme,
    paystack, planetscale, planetscaledb, portainer, posthog, postman, prefect,
    private_key, pubnub_publish, pubnub_subscribe, pulumi, ramp, razorpay, readme,
    rechargepayments, replicate, rubygems, salesforce, sendgrid, sendinblue,
    shopify, slack_access, slack_bot, slack_refresh, slack_user, slack_webhook,
    slack_workflow_webhook, sourcegraph, sourcegraphcody, squareapp, squareup,
    stripe, supabase, tailscale, tines_webhook, trufflehog, twilio, ubidots,
    voiceflow, web_url_with_credentials, zapierwebhook.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """

    topics: Optional[Dict[str, float]] = None
    r"""The topic of the classification.

    The current list can be obtained through the analyzers API, with the following
    conditions:
    1. Includes all analyzers EXCEPT those in the 'Modality' group
    2. Includes all analyzer detectors EXCEPT those in these groups:
    - 'Secrets'
    - 'Exploits'
    - 'Intent'
    - 'PIIs'
    - 'Malcontents'

    Example of topics: category/enterprise, category/healthcare, category/medical,
    category/personal, contains/blank_image, contains/certificate_signing_request,
    contains/database_url, contains/handwritten_text, contains/printed_text,
    contains/public_key, contains/ssh_public_key, contains/web_url,
    decoded/base32_data, decoded/base58_data, decoded/base64_data,
    decoded/base64_url_data, decoded/base85_data, decoded/hexadecimal_data,
    decoded/puny_code_data, decoded/quoted_printable_data,
    decoded/url_encoding_data, department/compliance, department/cybersecurity,
    department/finance, department/human_resources,
    department/information_technology, department/legal, department/logistics,
    department/marketing, department/performance_evaluation,
    department/product_management, department/project_management, department/sales,
    department/strategy, department/support, depict/check, depict/code,
    depict/document, depict/unclassified, depict/whiteboard, domain/commercial,
    domain/development, domain/financial, domain/general, domain/regulatory,
    domain/workforce, extracted/csv_cell_content, extracted/document_text_content,
    extracted/handwritten_text_content, extracted/pdf_text_content,
    extracted/pptx_text_content, extracted/spreadsheet_text_content,
    extracted/typed_text_content, timeframe/current_year, timeframe/future,
    timeframe/last_year, timeframe/next_year, timeframe/past,
    transcribed/audio_text_content.

    If a key is not present in the map that implies that entity was not detected. If
    an entity is detected then the value is a score. Here's how to interpret the
    scores:
    - confidence: 'low', 0 < score <0.33
    - confidence: 'medium', 0.33 <= score < 0.66
    - confidence: 'high', 0.66 <= score <= 1.
    """
